// Module: FindBestRmfName
// Author: Richard Saxton
//
// Description: 
//
// This class defines a section of an image and includes methods
// to manioulate it.
//
// Modifications: See ChangeLog
// 
// $Id: FindBestRmfName.cc,v 1.5 2006/09/14 13:08:43 rsaxton Exp $

// General classes
#include <cmath>

// SAS-specific classes
#include "errstr.h"
#include "Tracer.h"
#include "PixelIterator.h"
#include "ImagePixelIterator.h"
#include "TablePixelIterator.h"
#include "SourcePosition.h"
#include "DatafileAttributes.h"
#include "PosWCS.h"
#include "Xmm.h"
#include "DataFilter.h"
#include "DssBuilder.h"
#include "DataSubspace.h"
#include "Evaluator.h"
#include "metatask.h"
#include "attcalc.h"
#include "CalServer.h"
#include "CorePosData.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"
#include "NumericalRange.h"

// Local classes
#include "FindBestRmfName.h"


/* Constructor */
//FindBestRmfName::FindBestRmfName();
/* Destructor */
//FindBestRmfName::~FindBestRmfName()

/* Method: getName(sourceset) - finds the RMF corresponding to the spectrum */
string FindBestRmfName::getName( const string & sourceFile )
{
  PROC;

   string bestRmf = " ", patbit=" ", modebit=" ", posbit=" ";
   string intro=" ", datebit=" ";

 /* If any problems set a default string controlled by this bbolean */
   bool notfound = false;

 /* Open the input file as a dataset */
   DataSet *spectrumset = dataSetServer->open( sourceFile, DataSet::Read );
   Table * spectrumtable = spectrumset->table("SPECTRUM");

 /* Read header keywords */
   DatafileAttributes *fileAttribs = new DatafileAttributes(sourceFile);
   Xmm::Mode windowMode = fileAttribs->getWindowMode();

   
 /* Use rmfgen::CorePosData class to get the event grade */
   CorePosData * corePos = new CorePosData(spectrumset, spectrumtable,
                                                            fileAttribs);
   EventGrades eventGrades = corePos->patternCode();
           
   switch (fileAttribs->instrumentID())
    {
    case Xmm::EMOS1 :
    case Xmm::EMOS2 :
      {
     errstr << "Found event grade "<<eventGrades
            << Sas::message(Msg::App, Msg::Verbose);
           
       /* Get the pattern bits */
         switch (eventGrades)
          {
           case CombinedEventGrades::SINGLE :
            {
             patbit = "p0"; break;
            }
           case CombinedEventGrades::SDTQ : 
            {
             patbit = "pall"; break;
            }
           default:
            { notfound=true; }
          }

       /* Set the mode string */
         if ( windowMode == Xmm::FAST_UNCOMPRESSED ) // from somewhere
           { modebit = "tu"; }
         else
           { modebit = "im"; }
           
       errstr << "modebit "<<modebit
            << Sas::message(Msg::App, Msg::Verbose);
               
       /* Set the date string */
         STime numdate(fileAttribs->observationDate());   

         if ( numdate < STime("2000-07-15T00:00:00") )
           { datebit = "000_"; }
         else if  ( numdate < STime("2000-11-09T00:00:00") )
           { datebit = "110_"; }
         else if  ( numdate < STime("2001-04-18T00:00:00") )
           { datebit = "169_"; }
         else if  ( numdate < STime("2001-08-18T00:00:00") )
           { datebit = "255_"; }
         else if  ( numdate < STime("2001-09-26T00:00:00") )
           { datebit = "310_"; }
         else if  ( numdate < STime("2001-11-25T00:00:00") )
           { datebit = "330_"; }
         else if  ( numdate < STime("2002-05-01T00:00:00") )
           { datebit = "360_"; }
         else if  ( numdate < STime("2002-11-07T00:00:00") )
           { datebit = "439_"; }
         else 
           { datebit = "534_"; }

                errstr << "DateBit "<<datebit
            << Sas::message(Msg::App, Msg::Verbose);
            
      /* Instrument string */
         if (fileAttribs->instrumentID() == Xmm::EMOS1)
           { intro = "m1_";}
         else
           { intro = "m2_";}

         //bestRmf = intro + modebit + "_" + patbit + "_" + datebit + ".rmf";
         //bestRmf = intro + datebit + modebit + "_" + patbit + "_" + "v1.2.rmf";
         bestRmf = intro + datebit + modebit + "_" + patbit + ".rmf";
         break;
      }
    case Xmm::EPN   :
      {
       /* Get the pattern bits */
         switch (eventGrades)
          {
           case CombinedEventGrades::SINGLE :
            {
             patbit = "s";
             break;
            }
           case CombinedEventGrades::SD :
            {
             patbit = "sd"; break;
            }
           case CombinedEventGrades::DOUBLE : 
            {
             patbit = "d"; break;
            }
           default:
            { notfound=true; }
          }

       /* Set the mode string */
         switch (windowMode) 
          {
           case Xmm::PRIME_FULL_WINDOW :
            { 
             modebit = "ff20"; break;
            }
           case Xmm::PRIME_FULL_WINDOW_EXTENDED :
            { 
             modebit = "ef20"; break;
            }
           case Xmm::PRIME_SMALL_WINDOW:
            { 
             modebit = "sw20"; break;
            }
           case Xmm::PRIME_LARGE_WINDOW:
            { 
             modebit = "lw20"; break;
            }
           case Xmm::FAST_TIMING :
            { 
             modebit = "ti40"; break;
            }
           case Xmm::FAST_BURST :
            { 
             modebit = "bu23"; break;
            }                                        
           default:
            { notfound=true; }
         }
         intro = "epn_";

       /* Find the RAW-Y position string */
         int pnline = INT(floor(corePos->centre_raw().y() / 20.0));
         posbit = "Y" + toString(pnline);

         //bestRmf = intro + modebit + "_" + patbit + posbit + "_v6.7.rmf";
         bestRmf = intro + modebit + "_" + patbit + posbit + ".rmf";
         break;
      }        
    default:
      errstr << "Dont recognise the instrument "<<fileAttribs->instrument()
             << Sas::error("invalidInstrument");
    } // end of instrument case

   /* Set default string if any problems during the name construction */
    if (notfound) {bestRmf = "NoCannedMatrixForThisSpectrum";}

    errstr << "Will set canned RMF name to "<<bestRmf
           << Sas::message(Msg::App, Msg::Verbose);
    
   /* Close down and tidy up */
    delete corePos;
    delete fileAttribs; 
    dataSetServer->close(spectrumset);
        
    RETURN bestRmf;
}

